"""entity/relation schemas to store email in an cubicweb instance

:organization: Logilab
:copyright: 2006-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"


class Email(EntityType):
    """electronic mail"""
    subject   = String(fulltextindexed=True)
    date      = Datetime(description=_('UTC time on which the mail was sent'))
    messageid = String(required=True, indexed=True)
    headers   = String(description=_('raw headers'))
    
    sender     = SubjectRelation('EmailAddress', cardinality='1*')
    # an email with only Bcc is acceptable, don't require any recipients
    recipients = SubjectRelation('EmailAddress')
    cc         = SubjectRelation('EmailAddress')
    
    parts       = SubjectRelation('EmailPart', cardinality='*1', composite='subject')
    attachment  = SubjectRelation('File')
    
    reply_to    = SubjectRelation('Email', cardinality='?*')
    cites       = SubjectRelation('Email')
    in_thread   = SubjectRelation('EmailThread', cardinality='?*')


class EmailPart(MetaEntityType):
    """an email attachment"""
    permissions = {
        'read':   ('managers', 'users', 'guests',), # XXX if E parts X, U has_read_permission E
        'add':    ('managers', ERQLExpression('E parts X, U has_update_permission E'),),
        'delete': ('managers', ERQLExpression('E parts X, U has_update_permission E')),
        'update': ('managers', 'owners',),
        }
    
    content  = String(fulltextindexed=True)
    content_format = String(required=True, meta=True, maxsize=50)
    ordernum = Int(required=True)
    alternative = SubjectRelation('EmailPart', symetric=True)
    

class EmailThread(EntityType):
    """discussion thread"""
    title = String(required=True, indexed=True, fulltextindexed=True)
    see_also = SubjectRelation('EmailThread')
    forked_from = SubjectRelation('EmailThread', cardinality='?*')
    
class parts(RelationType):
    """"""
    fulltext_container = 'subject'
    
class sender(RelationType):
    """"""
    inlined = True
    
class in_thread(RelationType):
    """"""
    inlined = True
    
class reply_to(RelationType):
    """"""
    inlined = True

class generated_by(RelationType):
    """mark an entity as generated from an email"""
    cardinality = '?*'
    subject = ('TrInfo',)
    object = 'Email'

# if comment is installed
if 'Comment' in defined_types:
    class comment_generated_by(RelationDefinition):
        subject = 'Comment'
        name = 'generated_by'
        object = 'Email'
    

#class Role(Entity):
#    SubjectRelation('name', 'String')

#class PersonThreadRole(Entity):
    
    
