"""unit tests for email mbox import functionnality"""

from mx.DateTime import now
from StringIO import StringIO

from logilab.common.testlib import TestCase, unittest_main

from cubicweb.devtools.apptest import EnvBasedTC

from cubes.email.mboximport import MBOXImporter


class MBOXImporterTC(EnvBasedTC):

    def test_all(self):
        cursor = self.cursor()
        mi = MBOXImporter(cursor, self.schema)
        mi.import_mbox_stream(open('data/mbox'))
        self.assertEquals(sorted([(x, len(y)) for x, y in mi.created.items()]),
                                 [('email', 2), ('emailaddress', 4),
                                  ('emailpart', 5), ('emailthread', 2), ('file', 2)])
        self.assertEquals(mi.skipped, [])
        rset = self.execute('Any X ORDERBY S WHERE X is Email, X subject S')
        self.assertEquals(len(rset), 2)
        
        email = rset.get_entity(0, 0)
        self.assertEquals(email.subject, 'Re: [Python-projects] Pylint: Disable-msg for a block or statement?')
        self.assertEquals(email.sender[0].address, 'pink@odahoda.de')
        self.assertEquals(email.sender[0].alias, 'Benjamin Niemann')
        self.assertEquals([r.address for r in email.recipients], ['python-projects@logilab.org'])
        self.assertEquals([r.address for r in email.cc], [])
        self.assertEquals(email.in_thread[0].title, '[Python-projects] Pylint: Disable-msg for a block or statement?')
        self.assertEquals(sorted([(f.name, f.data_format) for f in email.attachment]),
                          [(u'astng.patch', u'text/x-diff'), (u'pylint.patch', u'text/x-diff')])
        assert not self.vreg.schema['parts'].inlined
        self.assertEquals(len(cursor.execute('Any P WHERE E parts P, E eid %s'%email.eid)), 2)
        self.assertEquals(len(email.parts), 2)
        part1, part2 = email.parts_in_order()
        self.assertEquals(part1.content_format,'text/plain')
        self.assertEquals(part2.content_format,'text/plain')
        self.assertEquals(part2.content, '''_______________________________________________
Python-Projects mailing list
Python-Projects@lists.logilab.org
http://lists.logilab.org/mailman/listinfo/python-projects''')

        email = rset.get_entity(1, 0)
        self.assertEquals(email.subject, '[Python-projects] pylint: False positive about field initialisation')
        self.assertEquals(email.sender[0].address, 'maarten.ter.huurne@philips.com')
        self.assertEquals(email.sender[0].alias, 'Maarten ter Huurne')
        self.assertEquals([r.address for r in email.recipients], ['python-projects@lists.logilab.org'])
        self.assertEquals([r.address for r in email.cc], [])
        self.assertEquals(email.in_thread[0].title, '[Python-projects] pylint: False positive about field initialisation')
        self.assertEquals([(f.name, f.data_format) for f in email.attachment], [])
        self.assertEquals(len(email.parts), 3)
        self.assertEquals(sorted([r.content_format for r in email.parts]), [u'text/html', u'text/plain', u'text/plain'])
        part1, part2 = email.parts_in_order()
        self.assertEquals(part1.content_format,'text/html')
        self.assertEquals(part2.content_format,'text/plain')
        self.assertEquals(part2.content, '''_______________________________________________
Python-Projects mailing list
Python-Projects@lists.logilab.org
http://lists.logilab.org/mailman/listinfo/python-projects''')
        part1, part2 = email.parts_in_order('text/plain')
        self.assertEquals(part1.content_format,'text/plain')
        self.assertEquals(part2.content_format,'text/plain')


    NOSUBJECT = """
From python-projects-bounces@lists.logilab.org  Tue Mar 28 17:06:41 2006
Return-Path: <python-projects-bounces@lists.logilab.org>
X-Original-To: sylvain.thenault@logilab.fr
Delivered-To: syt@logilab.fr
To: python-projects@lists.logilab.org
MIME-Version: 1.0
X-Mailer: Lotus Notes Release 6.0.3 September 26, 2003
Message-ID: <OFE9ED18DA.E3F2A23A-ONC125713F.00521FE5-C125713F.0052FE93@philips.com>
References: <xxx@blabla>
From: Maarten ter Huurne <MAARTEN.TER.HUURNE@PHILIPS.COM>
Date: Tue, 28 Mar 2006 17:05:23 +0200
X-MIMETrack: Serialize by Router on ehvrmh02/H/SERVER/PHILIPS(Release
	6.5.3FP1HF291 | September 19, 2005) at 28/03/2006 17:05:23,
	Serialize complete at 28/03/2006 17:05:23
X-BeenThere: python-projects@lists.logilab.org
X-Mailman-Version: 2.1.5
Precedence: list
Content-Type: multipart/mixed; boundary="===============1708445001=="
Mime-version: 1.0
Sender: python-projects-bounces@lists.logilab.org
Errors-To: python-projects-bounces@lists.logilab.org
X-Spambayes-Classification: ham; 0.00
Status: RO
Content-Length: 2915
Lines: 82

This is a multipart message in MIME format.
--===============1708445001==
Content-Type: multipart/alternative;
	boundary="=_alternative 0052FE92C125713F_="

This is a multipart message in MIME format.
--=_alternative 0052FE92C125713F_=
Content-Type: text/plain; charset="US-ASCII"

Hi,

On the following program:
===
class C:
    def __init__(self):
        self.set(self, 'abc')
    def set(self, value):
        self.__value = value
        self.__length = len(value)
===
pylint reports:
===
W0201:  5:C.set: Attribute '__value' defined outside __init__
W0201:  6:C.set: Attribute '__length' defined outside __init__
===

Although strictly speaking they are indeed defined outside __init__, these 
fields are guaranteed to be initialised when an object of type C is 
constructed. It would be useful if pylint could recognise situations like 
this one and not issue this warning.

Bye,
                Maarten

--=_alternative 0052FE92C125713F_=
Content-Type: text/html; charset="US-ASCII"


<br><font size=2 face="sans-serif">Hi,</font>
<br>
<br><font size=2 face="sans-serif">On the following program:</font>
<br><font size=2 face="sans-serif">===</font>
<br><font size=2 face="sans-serif">class C:</font>
<br><font size=2 face="sans-serif">&nbsp; &nbsp; def __init__(self):</font>
<br><font size=2 face="sans-serif">&nbsp; &nbsp; &nbsp; &nbsp; self.set(self,
'abc')</font>
<br><font size=2 face="sans-serif">&nbsp; &nbsp; def set(self, value):</font>
<br><font size=2 face="sans-serif">&nbsp; &nbsp; &nbsp; &nbsp; self.__value
= value</font>
<br><font size=2 face="sans-serif">&nbsp; &nbsp; &nbsp; &nbsp; self.__length
= len(value)</font>
<br><font size=2 face="sans-serif">===</font>
<br><font size=2 face="sans-serif">pylint reports:</font>
<br><font size=2 face="sans-serif">===</font>
<br><font size=2 face="sans-serif">W0201: &nbsp;5:C.set: Attribute '__value'
defined outside __init__</font>
<br><font size=2 face="sans-serif">W0201: &nbsp;6:C.set: Attribute '__length'
defined outside __init__</font>
<br><font size=2 face="sans-serif">===</font>
<br>
<br><font size=2 face="sans-serif">Although strictly speaking they are
indeed defined outside __init__, these fields are guaranteed to be initialised
when an object of type C is constructed. It would be useful if pylint could
recognise situations like this one and not issue this warning.</font>
<br>
<br><font size=2 face="sans-serif">Bye,</font>
<br><font size=2 face="sans-serif">&nbsp; &nbsp; &nbsp; &nbsp; &nbsp;
&nbsp; &nbsp; &nbsp; Maarten</font>
<br>
--=_alternative 0052FE92C125713F_=--

--===============1708445001==
Content-Type: text/plain; charset="us-ascii"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Disposition: inline

_______________________________________________
Python-Projects mailing list
Python-Projects@lists.logilab.org
http://lists.logilab.org/mailman/listinfo/python-projects
--===============1708445001==--
"""

    def test_no_subject(self):
        cu = self.env.cnx.cursor() # don't use self.cursor() to have no request set
        mi = MBOXImporter(cu, self.schema)
        mi.import_mbox_stream(StringIO(self.NOSUBJECT))
        rset = self.execute('Any X ORDERBY S WHERE X is Email, X subject S')
        self.assertEquals(len(rset), 1)
        email = rset.get_entity(0, 0)
        self.assertEquals(email.subject, '(no subject)')
        self.assertEquals(email.references(), set(('<xxx@blabla>',)))

    def test_double_import(self):
        cu = self.env.cnx.cursor() # don't use self.cursor() to have no request set
        mi = MBOXImporter(cu, self.schema)
        mi.import_mbox_stream(StringIO(self.NOSUBJECT))
        mi.import_mbox_stream(StringIO(self.NOSUBJECT))
        rset = self.execute('Any X ORDERBY S WHERE X is Email, X subject S')
        self.assertEquals(len(rset), 1)

    def test_address_detection(self):
        # test both case normalization and canonicalization of email address
        eid1 = self.execute('INSERT EmailAddress X: X address "maarten.ter.huurne@philips.com"')[0][0]
        eid2 = self.execute('INSERT EmailAddress X: X address "maarten@philips.com", X canonical TRUE')[0][0]
        self.execute('SET X identical_to Y WHERE X eid %s, Y eid %s' % (eid1, eid2))
        #self.assertEquals(env.vreg.schema['identical_to'].symetric, True)
        #ea1 = cursor.execute('Any X WHERE X eid %s' % eid2).get_entity(0, None)
        #self.assertEquals(ea1.canonical, True)
        #testrset = cursor.execute('Any X WHERE X canonical FALSE')
        #self.assertEquals(testrset[0][0], eid1)
        #testrset = cursor.execute('Any X WHERE X canonical TRUE')
        #self.assertEquals(testrset[0][0], eid2)
        #testrset = cursor.execute('Any X WHERE X identical_to Y, X canonical TRUE, Y eid %(y)s', {'y': eid1})
        #self.assertEquals(testrset[0][0], eid2)
        cu = self.env.cnx.cursor() # don't use self.cursor() to have no request set
        mi = MBOXImporter(cu, self.schema)
        mi.import_mbox_stream(StringIO(self.NOSUBJECT))
        rset = self.execute('Any X ORDERBY S WHERE X is Email, X subject S')
        self.assertEquals(len(rset), 1)
        email = rset.get_entity(0, 0)
        self.assertEquals(email.sender[0].address, 'maarten@philips.com')
        
        

        
if __name__ == '__main__':
    unittest_main()

